#include "..\include\internls\evrythng.h"

#include "..\include\swintrac\swintrac.h"
#include "..\include\detecuse\detecuse.h"
#include "..\include\motrstrg\motrstrg.h"

// Module fr die automatische Justage
#include  "..\include\autojust\autojust.h"

#include  "..\include\autojust\matrix.h"
#include  "..\include\autojust\transfrm.h"
#include  "..\include\autojust\m_justag.h"

#pragma hdrstop

extern LPDList lpDList;

//! definiert in m_justage.cpp
extern int nMotorDF, nMotorTL, nMotorCC;
extern bool bWriteLogfile;

const double GOLD = 0.61803;

// *************************************************************************
// ***************** Transformationsklasse *********************************
// zur Durchfhrung von Koordinatentransformationen


// Kontruktor
TransformationClass::TransformationClass(void)
{
  TVektor tempVektor(0,0,0);
  PosVektor = tempVektor;
  anzahl_koordinatentrafos = 0;
  bIntensityTooHigh = false;

    // Speicher fr temporaere Puffer reservieren
    buf = new char[1024];
    buf1= new char[1024];
  
  if ( buf == 0 || buf1 == 0 )
  {
    MessageBox(GetFocus(), "TransformationClass::GetIntensityOnPosition: \
      Fehler bei Speicherreservierung", "xalloc", MB_OK);
    PostQuitMessage(-1);
  }

}

// Destruktor
TransformationClass::~TransformationClass(void)
{
  delete[] buf;
  delete[] buf1;
}

// Initialisierung der Intervallgrenzen der Motoren
bool TransformationClass::Initialize(double MaxDF,double MaxTL,double MaxCC)
{
  double fPosition, fTemp;
  // Ermittlung der aktuellen Motorpositionen und
  // Festlegen der Intervallgrenzen zur Justage
  // Antrieb:Tilt
  // setzen der aktuellen Antriebsachse
  mlSetAxis(nMotorTL);
  // aktuelle Position des Antriebs bestimmen
  mGetDistance(fPosition);
  Wertebereich.OrigTL = fPosition;
  // Tilt: Wertebereich +/- 20 Minuten
  Wertebereich.MinTL = (-1)*MaxTL;
  Wertebereich.MaxTL = MaxTL;
  // Antrieb:Beugung fein
  mlSetAxis(nMotorDF);
  mGetDistance(fPosition);
  Wertebereich.OrigDF = fPosition;
  // DF: Wertebereich +/- 200 Sekunden
  Wertebereich.MinDF = (-1)*MaxDF;
  Wertebereich.MaxDF = MaxDF;
  // Antrieb:Kollimator
  mlSetAxis(nMotorCC);
  mGetDistance(fPosition);
  Wertebereich.OrigCC = fPosition;
  // CC: Wertebereich +/- 100 Mikrometer
  Wertebereich.MinCC = (-1)*MaxCC;
  Wertebereich.MaxCC = MaxCC;
  // Vergleich mit Softwareschranken und ggf. nderung der Wertebereiche
  if ((fTemp=mlGetValue(nMotorTL,MinDistance)) > Wertebereich.MinTL +
                                                           Wertebereich.OrigTL)
  { Wertebereich.MinTL = fTemp - Wertebereich.OrigTL; }
  if ((fTemp=mlGetValue(nMotorTL,MaxDistance)) < Wertebereich.MaxTL +
                                                           Wertebereich.OrigTL)
  { Wertebereich.MaxTL = fTemp - Wertebereich.OrigTL; }
  if ((fTemp=mlGetValue(nMotorDF,MinDistance)) > Wertebereich.MinDF +
                                                           Wertebereich.OrigDF)
  { Wertebereich.MinDF = fTemp - Wertebereich.OrigDF; }
  if ((fTemp=mlGetValue(nMotorDF,MaxDistance)) < Wertebereich.MaxDF +
                                                           Wertebereich.OrigDF)
  { Wertebereich.MaxDF = fTemp - Wertebereich.OrigDF; }
  if ((fTemp=mlGetValue(nMotorCC,MinDistance)) > Wertebereich.MinCC +
                                                           Wertebereich.OrigCC)
  { Wertebereich.MinCC = fTemp - Wertebereich.OrigCC; }
  if ((fTemp=mlGetValue(nMotorCC,MaxDistance)) < Wertebereich.MaxCC +
                                                           Wertebereich.OrigCC)
  { Wertebereich.MaxCC = fTemp - Wertebereich.OrigCC; }
  return true;
}


// Berechnung der Weltkoordinaten eines Punktes im akt. KS
TVektor TransformationClass::translate_from_worldpoints(TVektor OrigVektor)
{
  // 1. vektor homogen machen
  OrigVektor.mache_homogen();

  // 2. (HIN-Transformation)
  // Multiplikation des homog. Vektors mit den Trafo-Matrizen von Anfang an
  // (1. bis letzte Trafo-Matrix durchlaufen)
  for (unsigned i=1; i <= trafo_hin.elementanzahl(); i++)
  {
    OrigVektor = trafo_hin.zeige(i) * OrigVektor;
  }

  // 3. Rckgabe des Vektors in kartesischer Form
  return OrigVektor.mache_kartesisch();
}

// Umrechnung eines Punktes im Welt-KS in Koordinaten im akt. KS
TVektor TransformationClass::translate_to_worldpoints(TVektor OrigVektor)
{
  // 1. vektor homogen machen
  OrigVektor.mache_homogen();

  // 2. (RCK-Transformation)
  //    Multiplikation des homog. Vektors mit den Trafo-Matrizen
  //    vom Ende der Liste bis zum Anfang
  for (unsigned i=trafo_rueck.elementanzahl(); i > 0; i--)
  {
    OrigVektor = trafo_rueck.zeige(i) * OrigVektor;
  }

  // 3. Rckgabe des Vektors in kartesischer Form
  return OrigVektor.mache_kartesisch();
}


// bersetzung der Originalintervalle in akt. Koordinaten
TMotorPositionsWerte TransformationClass::translate_PosBorders(void)
{
  TVektor Punkt3D(3);
  TMotorPositionsWerte Intervalle;
  double X,Y,Z;

  // bersetzung der Intervalle in akt. Koordinaten
  // unbedingt mit Vergleich der Softwareschranken !!!
  // d.h. notwendige Rckrechnung (translate_to_worldpoints)
  // der akt. Schranken in Weltkoordinaten
  // aktSchranke(x,y,z) -> weltkoord_aktSchranke(x_w,y_w,z_w)
  // x_w-Werte mit Wertebereich.Min/MaxTL bzw. Softwareschranke vergl. usw.

  // Intervallgrenzen aus Wertebereich fr TL umrechnen
  Punkt3D.set_XYZ(Wertebereich.MinTL, 0, 0);
  Punkt3D = translate_from_worldpoints(Punkt3D);
  Punkt3D.get_XYZ(Intervalle.MinTL, Y, Z);
  Punkt3D.set_XYZ(Wertebereich.MaxTL, 0, 0);
  Punkt3D = translate_from_worldpoints(Punkt3D);
  Punkt3D.get_XYZ(Intervalle.MaxTL, Y, Z);

  // Intervallgrenzen fr DF umrechnen
  Punkt3D.set_XYZ(0, Wertebereich.MinDF, 0);
  Punkt3D = translate_from_worldpoints(Punkt3D);
  Punkt3D.get_XYZ(X, Intervalle.MinDF, Z);
  Punkt3D.set_XYZ(0, Wertebereich.MaxDF, 0);
  Punkt3D = translate_from_worldpoints(Punkt3D);
  Punkt3D.get_XYZ(X, Intervalle.MaxDF, Z);

  // Intervallgrenzen fr TL umrechnen
  Punkt3D.set_XYZ(0, 0, Wertebereich.MinCC);
  Punkt3D = translate_from_worldpoints(Punkt3D);
  Punkt3D.get_XYZ(X, Y, Intervalle.MinCC);
  Punkt3D.set_XYZ(0, 0, Wertebereich.MaxCC);
  Punkt3D = translate_from_worldpoints(Punkt3D);
  Punkt3D.get_XYZ(X, Y, Intervalle.MaxCC);

  return Intervalle;
}


float TransformationClass::GetIntensityOnPosition(unsigned nMeasureCount)
// fhrt die zu bersetzenden (TRAFO Klasse) Motorpositionen des Vektors an
// und gibt Intensitt zurck
// muss alle 3 Motorachsen anfahren
// 1. translate_to_worldpoints(akt.anzufahrenderPosVektor)
// 2. worldkoordinaten-Vektor.get_XYZ(&x,&y,&z)
// 3. nMotorTL mMoveToDistance (x + Wertebereich.OrigTL) weil Ursprung (0,0,0)
//    nMotorDF mMoveToDistance (y + Wertebereich.OrigDF)
//    nMotorCC mMoveToDistance (z + Wertebereich.OrigCC)
// 4. Rckgabe der Intensitt
{
  float fIntensity;
  double x,y,z,x2,y2,z2;

  // Infos frs Logging aufbereiten
  PosVektor.get_XYZ(x2, y2, z2);
  sprintf(buf,"Move2Dist:aktVektor (%.2f,%.2f,%.2f) -> ",x2,y2,z2);

  // Umrechnung des PosVektors in Weltkoordinaten
  PosVektor = translate_to_worldpoints(PosVektor);

  // Infos frs Logging aufbereiten
  PosVektor.get_XYZ(x, y, z);
  sprintf(buf1,"weltVektor (%.2f,%.2f,%.2f)[%d.KS]",x,y,z,
                get_koordinatentrafos());
  strcat (buf,buf1);
  // ins Logfile schreiben
  if (bWriteLogfile) WriteToJustageLog(buf);

  // Auslesen der Weltkoordinaten des PosVektors
  PosVektor.get_XYZ(x, y, z);

  // setzen der Antriebsachse TL und Anfahren der TL-Position
  mlSetAxis(nMotorTL);
  mMoveToDistance (x + GetOrigPosBorders().OrigTL);
  // setzen der Antriebsachse DF und Anfahren der DF-Position
  while (!mIsMoveFinish());
  mlSetAxis(nMotorDF);
  mMoveToDistance (y + GetOrigPosBorders().OrigDF);
  // setzen der Antriebsachse CC und Anfahren der CC-Position
  while (!mIsMoveFinish());
  mlSetAxis(nMotorCC);
  mMoveToDistance (z + GetOrigPosBorders().OrigCC);

  // Warten bis Bewegung beendet ist
  while (!mIsMoveFinish());

  fIntensity = MeasureIntensity(nMeasureCount);

  // Ausgabe in Statusleiste
  sprintf(buf,"I: %.0f", fIntensity);
  SetInfo(buf);

  // ins Logfile schreiben
  strcat(buf, "\n");
  if (bWriteLogfile) WriteToJustageLog(buf);
  // Werte fr den PosVektor zurckschreiben
  PosVektor.set_XYZ(x2, y2, z2);
  return fIntensity;
}


int TransformationClass::Goldener_Schnitt(TLaufachse achse,
                     double & intervall_li, double & intervall_re,
                     unsigned nMeasureCount)
{
  double a,b,u,v,fu,fv, x, y, z;
  int step;

  a=intervall_li;
  b=intervall_re;

  u=GOLD*a+(1-GOLD)*b;
  v=(1-GOLD)*a+GOLD*b;

  // Komponenten des Positionsvektors auslesen
  PosVektor.get_XYZ(x,y,z);

  // # Logging #
  sprintf(buf,">>GoldenerSchnitt: PosVektor (%.2f,%.2f,%.2f)\n",x,y,z);
  if (bWriteLogfile) WriteToJustageLog(buf);

  switch (achse)
  {
    case Laufachse_X: // Setzen der jew. Komponente im PosVektor
                      PosVektor.set_XYZ(u,y,z);
                      // Bestimmung der Intensitt
                      fu=GetIntensityOnPosition(nMeasureCount);
                      PosVektor.set_XYZ(v,y,z);
                      fv=GetIntensityOnPosition(nMeasureCount);
                      break;
    case Laufachse_Y: PosVektor.set_XYZ(x,u,z);
                      fu=GetIntensityOnPosition(nMeasureCount);
                      PosVektor.set_XYZ(x,v,z);
                      fv=GetIntensityOnPosition(nMeasureCount);
                      break;
    case Laufachse_Z: PosVektor.set_XYZ(x,y,u);
                      fu=GetIntensityOnPosition(nMeasureCount);
                      PosVektor.set_XYZ(x,y,v);
                      fv=GetIntensityOnPosition(nMeasureCount);
                      break;
    default:
    return -1;
  }

  step=2;

  while ( (fabs(a-b) > toleranz) )
  {
    if (fu > fv)
    {
      b=v; // re. Intervallgrenze nach links verschieben
      v=u;
      u=GOLD*a+(1-GOLD)*b; // neue li. Intervall-Position
      fv=fu;
      // Bestimmung des neuen Funktionswertes  (Intensitt)
      switch (achse)
      {
        case Laufachse_X: PosVektor.set_XYZ(u,y,z);
                          fu=GetIntensityOnPosition(nMeasureCount);
                          break;
        case Laufachse_Y: PosVektor.set_XYZ(x,u,z);
                          fu=GetIntensityOnPosition(nMeasureCount);
                          break;
        case Laufachse_Z: PosVektor.set_XYZ(x,y,u);
                          fu=GetIntensityOnPosition(nMeasureCount);
                          break;
       }
      step++;
    }
    else // wenn (fu <= fv)
    {
      a=u; // li. Intervallgrenze nach rechts verschieben
      u=v;
      v=(1-GOLD)*a+GOLD*b; // neue re. Intervall-Position
      fu=fv;
      // Bestimmung des neuen Funktionswertes (Intensitt)
      switch (achse)
      {
        case Laufachse_X: PosVektor.set_XYZ(v,y,z);
                          fv=GetIntensityOnPosition(nMeasureCount);
                          break;
        case Laufachse_Y: PosVektor.set_XYZ(x,v,z);
                          fv=GetIntensityOnPosition(nMeasureCount);
                          break;
        case Laufachse_Z: PosVektor.set_XYZ(x,y,v);
                          fv=GetIntensityOnPosition(nMeasureCount);
                          break;
       }
      step++;
    }
  }
  intervall_li=u;
  intervall_re=v;
  return step;
}

float TransformationClass::MeasureIntensity(unsigned nMeasureCount)
{
  float fIntensity, fIntensity_temp, ftemp, fMedian;
  unsigned counter=0;
  float *Medianliste;


    // Speicher fr Medianliste reservieren
    Medianliste = new float[nMeasureCount];
  if (Medianliste == 0)
  {
    MessageBox(GetFocus(), "MeasureIntensity: Fehler bei Speicherreservierung",
               "xalloc", MB_OK);
    PostQuitMessage(-1);
  }
   // aktuelles Zhlergert setzen
  TDevice *Detektor = lpDList->DP();
  do
  {
    // Zhler starten
    Detektor->MeasureStart();

    // Auslesen der Detektorkarte veranlassen
    // im Fehlerfall muss die Messung neugestartet werden
    while (Detektor->PollDevice() != R_MeasOk)
    {
      // Messung neustarten
      // Detektor->MeasureStop();
     SetInfo("Warte auf Messwerte...");
     MessageBeep(0);
     //Detektor->MeasureStart();
    }
    // Intensitt auslesen (Membervariable fIntensity von Sensor)
    while (Detektor->GetData(fIntensity) != R_OK)
    {
      SetInfo("Hole Daten vom Counter...");
    }

    // erfolgreiche Messung
    counter++;
    
    // Medianliste erweitern & sortieren *****
    fIntensity_temp=fIntensity;

    if (fIntensity > 100000.0)
    {
      if (!bIntensityTooHigh)
      {
        MessageBox(GetFocus(), "MeasureIntensity: Die Maximalanzahl von 100.000\
               \nDetektorcounts wurde berschritten.\n \
               \nDie 'Automatische Justage' wird abgebrochen. \
               \nBitte regeln Sie die Spannung des Rntgengenerators nach.\n",
               "berschreitung des Detektormaximums", MB_OK);
        bIntensityTooHigh=true;
      }
    }

    // noch keine Messwerte in der Liste
    if ((counter-1) == 0)
    {
      Medianliste[0] = fIntensity_temp;
    }
    // Messwertliste besteht schon
    else
    {
      // solange bis alle bisherigen Messwerte durchgelaufen wurden
      for (int i=0; i < counter-1; i++)
      {
        // kleine Elemente zuerst einsortieren
        if(fIntensity_temp < Medianliste[i])
        {
          // Wert an der akt. Stelle merken
          ftemp=Medianliste[i];
          // Wert an der akt. Stelle durch Messwert ersetzen
          Medianliste[i]=fIntensity_temp;
          // Trick, um die weiteren Elemente durchzusortieren
          fIntensity_temp=ftemp;
        }
      }
      // brig gebliebener Wert wird ans Ende angehngt
      Medianliste[counter-1]=fIntensity_temp;
    }

    // Medianbestimmung
    // ungerade Anzahl von Messungen
    if((counter%2)==1)
    {
      if(counter==1)
      {
        // einziges Element
        fMedian=Medianliste[counter-1];
      }
      else
      {
        // mittleres Element der Medianliste
        fMedian=Medianliste[((counter+1)/2)-1];
      }
    }
    // gerade Anzahl von Messungen
    else
    {
      // Mittelwert der beiden mittleren Elemente
      fMedian=(Medianliste[(counter/2)-1]+Medianliste[((counter/2)+1)-1])/2;
    }
    // Infos ins Logfile schreiben
    if (bWriteLogfile)
    {
      sprintf(buf,"\r\nM:%d I:%.2f Median:%.2f ",counter,fIntensity,fMedian);
      WriteToJustageLog(buf);
    }

  } while(counter < nMeasureCount);

  delete[] Medianliste;

  return fMedian;
}

// Durchfhrung einer kompletten Koordinatentransformation um best. Achse
bool TransformationClass::KoordinatenTransformation(TDrehachse achse, \
                                                    TVektor VerschiebeVektor)
{
  TVektor Einheitsvektor(3);
  TMatrix Matrix;
  double winkel;

  switch (achse)
  {
    case Drehachse_X: // Einheitsvektor der Z-Achse
                      Einheitsvektor.set_XYZ(0,0,1); break;

    case Drehachse_Z: // Einheitsvektor der X-Achse
                      Einheitsvektor.set_XYZ(1,0,0); break;

    // bei Automatischer Justage wird nur um X bzw. Z gedreht
    default:
    return false;
  }

  // Berechnung des Winkels von Vektor zur Koordinatenachse (Einheitsvektor)
  winkel = VerschiebeVektor.winkel(Einheitsvektor);

  // Ausfhrung der jew. KS-Transformation (Hin - und Rck)
  // Drehung um X-Achse
  if (achse == Drehachse_X)
  {
    // Abspeichern der Hintransformation
    Matrix.transformiere(XYZ, (-1)*VerschiebeVektor, -winkel, 0, 0);
    trafo_hin.push(Matrix);
    // Abspeichern der Rcktransformation
    Matrix.transformiere(ZYX, VerschiebeVektor, winkel, 0, 0);
    trafo_rueck.push(Matrix);
  }
  // Drehung um Z-Achse
  else if (achse == Drehachse_Z)
  {
    // Abspeichern der Hintransformation
    Matrix.transformiere(XYZ, (-1)*VerschiebeVektor, 0, 0, -winkel);
    trafo_hin.push(Matrix);
    // Abspeichern der Rcktransformation
    Matrix.transformiere(ZYX, VerschiebeVektor, 0, 0,winkel);
    trafo_rueck.push(Matrix);
  }

  anzahl_koordinatentrafos ++;

  // # Logging #
  sprintf(buf,"%d. KS-Drehung: Winkel= %.2f Grad\n",anzahl_koordinatentrafos,
               winkel*180/M_PI);
  if (bWriteLogfile) WriteToJustageLog(buf);

  return true;
}

void TransformationClass::DFCorrection(unsigned nMeasureCount, double & fDFPos,
                                       float & fInt)
{
  double a,b,u,v,fu,fv;
  double fPosition, toleranz=.1;

  // Intervallgrsse selbst angeben
  // Bereich kann klein sein (DF: sek)
  a=-5;
  b=5;

  u=GOLD*a+(1-GOLD)*b;
  v=(1-GOLD)*a+GOLD*b;

  // Position festlegen
  mlSetAxis(nMotorDF);
  mGetDistance(fPosition);

  mMoveToDistance(u+fPosition);
  while (!mIsMoveFinish());
  fu=MeasureIntensity(nMeasureCount);
  mMoveToDistance(v+fPosition);
  while (!mIsMoveFinish());
  fv=MeasureIntensity(nMeasureCount);

  while ((fabs(a-b) > toleranz))
  {
    if (fu > fv)
    {
      b=v; // re. Intervallgrenze nach links verschieben
      v=u;
      u=GOLD*a+(1-GOLD)*b; // neue li. Intervall-Position
      fv=fu;
      // Bestimmung des neuen Funktionswertes (Intensitt)
      mMoveToDistance(u+fPosition);
      while (!mIsMoveFinish());
      fu=MeasureIntensity(nMeasureCount);
    }
    else // if (fu <= fv)
    {
      a=u; // li. Intervallgrenze nach rechts verschieben
      u=v;
      v=(1-GOLD)*a+GOLD*b; // neue re. Intervall-Position
      fu=fv;
      // Bestimmung des neuen Funktionswertes (Intensitt)
      mMoveToDistance(v+fPosition);
      while (!mIsMoveFinish());
      fv=MeasureIntensity(nMeasureCount);
    }
  }
  // Intensitt & Position zurckliefern
  fInt=fv;
  fDFPos=v+fPosition;
  // Position angefahren
}

