#include  "..\include\internls\evrythng.h"
#pragma hdrstop

#include "..\include\swintrac\swintrac.h"
#include "..\include\datavisa\datavisa.h"

// m_curve.cpp
// modul for the data base
// (C) 1993,1994 by Heiko Damerow MPG AG "Roentgenbeugung"
// Klassen und Instanzen fuer die Datenhaltung

//! kommentiert von diffr.gruppe

#define CINT  const int
//! Anzahl maximal speicherbarer Kurven
const int TDataBase::nMaxCurve = 5000;
//! gibt Anzahl der Punkte an, um die die Kurve erweitert wird
static CINT GrowUnit = 29000; //50;
//! Anzahl exist. Kurven auf ini.wert
int TCurve::LastId = -1;
//! scheinbar ueberfluessig
extern BOOL bModulLoaded;

LPDataBase lpDataBase;

//*****************************************************************************
//****************    Module TDataBase  ***************************************
//*****************************************************************************
//! konstruktor der Datenbasis
TDataBase::TDataBase( void )
{
  //! Indizes auf Initialwerte setzen + Liste mit 1001 Kurven anlegen +
  //! 0. Kurve mit 100 Punkten erstellen + Scanreportkurve auf 50 Punkte init.
  nMaxCurveIdx = 0;
  nActualIdx = 0;
  CurveList = new TCurve * [ nMaxCurve ];
  CurveList[ 0 ] = new TCurve( 100 );
  ScanReport = new TCurve( 50 );
};

//! Destruktor
TDataBase::~TDataBase( void )
{
  //! loeschen aller Kurven der Datenbasis
  TDataBase::New();
  delete CurveList[ 0 ];
  delete CurveList;
  delete ScanReport;
};

//! alle Kurven der Datenbasis loeschen und alle Indizes auf ini.werte
//! Rueckkehrcode immer true
BOOL TDataBase::New( void )
{
  //! Kurven von der letzten beginnend bis zur 1. loeschen
  nActualIdx = nMaxCurveIdx;
  while ( 0 < nActualIdx )
			delete CurveList[ nActualIdx-- ];
  //! Datenbasisindezes zuruecksetzen + 0. Kurve auf init.werte zuruecksetzen +
  //! scanreport auf ini.werte zuruecksetzen
  nMaxCurveIdx = 0;
  nActualIdx = 0;
  CurveList[ 0 ] ->New();
  ScanReport->New();
  return TRUE;
};

//! fuegt eine Kurve der angegebenen Groesse(1.Param.) zur Datenbasis hinzu
//! Rueckkehrcode false wenn max. Kurvenanzahl erreicht sonst true
BOOL TDataBase::AddCurve( int size )
{
  //! test ob max. anzahl speicherbarer Kurven erreicht wenn ja dann Fehler
  //! setzen + abbruch
  if ( nMaxCurveIdx + 2 > nMaxCurve )
  {
			eFault = MaxScanExceeded;
			return FALSE;
  }
  //! erstellt Kurve mit naechstgroesserem Index und angegeb. Anz. von Punkten
  CurveList[ ++nMaxCurveIdx ] = new TCurve( size + 1 );
  //! wenn erfolgreich abbruch sonst Fehler setzen
  if ( NULL != CurveList[ nMaxCurveIdx ] )
			return TRUE;
  eFault = NoMemory;
  return FALSE;
};

//! neu akk
//! entfernt die letze Kurve(dem Index nach) in der Datenbasis
//! Rueckkehrcode FALSE wenn Datenbasis leer
//! sonst TRUE
BOOL TDataBase::DelCurve( void )
{
 if (nMaxCurveIdx == 0)
  return FALSE;
 if (nActualIdx == nMaxCurveIdx)
  nActualIdx--;
 delete CurveList[ nMaxCurveIdx-- ];
 return TRUE;
}

//! setzt Kurvenindex auf angegebenen wert(1.Param.) wenn moeglich ansonsten
//! auf den entsprechenden Grenzwert
void TDataBase::SetCIdx( int idx )
{
  nActualIdx = max( 0, min( nMaxCurveIdx, idx ) );
};

//! liefert Kurve mit angegebenen Index(1.Param.) als Rueckkehrcode zurueck
//! wenn moeglich ansonsten naechstgelegene gueltige Kurve
TCurve *TDataBase::GetCurve( int idx )
{
  if ( idx < 0 )
			return CurveList[ 0 ];
  if ( idx > nMaxCurveIdx )
			return CurveList[ nMaxCurveIdx ];
  return CurveList[ idx ];
};

//*****************************************************************************
//****************    Module TCurve   *****************************************
//*****************************************************************************

//! in diesem Objekt werden die Messwerte eines Scans abgelegt
//! im stepscan
//! x-Wert = motor winkel (standard/continuous)
//!        = Omegamotorwinkel (Omega2Theta)
//! y-wert = gemessene intensitaet des Detektors in dieser Position
//! z-wert = 0 (standard/continuous) kein monitor ausgewaehlt
//!        = Monitorintensitaet (standard) monitor ausgewaehlt
//!        = Thetamotorwinkel (Omega2Theta)
//! im areascan
//!?       ...
//! Parameter 'npoints' gibt die Anzahl der Messpunkte der Kurve an die
//! 'vorlaeufig' geplant sind (koennen spaeter noch erweitert werden)
TCurve::TCurve( int npoints )
{
  //! nicht von diffr.gruppe auskommentiert
  //! long      retval;

  //! alt // Anzahl der Werte je Punkt
  //! initialisiert Kurve
  ActualIdx = -1;
  MaxPointIdx = -1;
  MaxMemIdx = -1;
  eOrder = X_Sorted;
  bEnd        = FALSE;
  bStreamOpen = FALSE;
  //! erhoeht Zaehler fuer Anzahl der im Speicher vorhandenen Kurven und weist
  //! ihn dieser Kurve als Ident. zu + legt Anzahl der max. Kurvenpunkte fest
  TCurve::LastId++;
  Id = TCurve::LastId;
  MaxMemIdx = max( GrowUnit, npoints + 2 );
  //! nicht von diffr.gruppe auskommentiert
  //! retval  = GlobalCompact(MaxMemIdx * sizeof(CPoint));
  //! GlobalCompact hat keine Bedeutung bei Win32API

  //! versucht speicherplatz fuer die gewuenscht Punkteanzahl zu reservieren
  GlobalCompact( MaxMemIdx * sizeof( CPoint ) );
  hMemX = GlobalAlloc( GHND, MaxMemIdx * sizeof( CPoint ) );
  //! wenn nicht moeglich Fehlermeldung + abbruch
  if ( !hMemX )
  {
#ifdef GermanVersion
			MessageBox( GetFocus(), "Kein Speicher TCurve", "Fehler",
						MB_OK | MB_ICONSTOP );
#else
			MessageBox( GetFocus(), "No Memory TCurve", "Failure",
						MB_OK | MB_ICONSTOP );
#endif
			MaxMemIdx = -1;
			return ;
  }
  //! min und max werte zu x,y und z initialisieren
  Max[ 0 ] = -( Min[ 0 ] = 1000000.0 );
  Max[ 1 ] = -( Min[ 1 ] = 1000000.0 );
  Max[ 2 ] = -( Min[ 2 ] = 1000000.0 );
//  MessageBox(GetFocus(),"nach","TCurve-konstr",MBINFO);
};


//! Destruktor
TCurve::~TCurve( void )
{
  //! loeschen der Kurvenzusatz inform. (min,max...)
  New();
  //! alt        // Gibt Segment entgueltig frei
  //! reduziert Zaehler fuer Anzahl der im Speicher vorhandenen Kurven
  //! gibt speicherplatz  x,y,z und v wieder frei
  TCurve::LastId--;
  MaxMemIdx = -1;
  GlobalFree( hMemX );
};


//! setzt kurve auf Initialwerte zurueck
//! Rueckkehrcode immer true
BOOL TCurve::New( void )
{
  //! ruecksetzen der indizes
  ActualIdx = -1;
  MaxPointIdx = -1;
  bEnd        = FALSE;

  //! zuuecksetzen des reserv. speichers auf die groesse GrowUnit ( = 50)
  GlobalFree(hMemX);
  GlobalCompact(GrowUnit * sizeof (CPoint));
  hMemX = GlobalAlloc(GHND,GrowUnit * sizeof (CPoint));
  if (! hMemX)
	 return FALSE;
  MaxMemIdx = ( int ) GrowUnit - 1;
  --MaxMemIdx;
  //! zuruecksetzen der max, min
  Max[ 0 ] = -( Min[ 0 ] = 1000000.0 );
  Max[ 1 ] = -( Min[ 1 ] = 1000000.0 );
  Max[ 2 ] = -( Min[ 2 ] = 1000000.0 );
  return TRUE;
};


//! setzt den internen Index des aktuellen Punktes (Tripels) um 1 zurueck,
//! sofern dies moeglich ist und setzt 'Kurvenstatus' auf nicht fertig
void TCurve::BackStep( void )
{
  bEnd = FALSE;
  if ( ActualIdx > 0 )
			--ActualIdx;
};


//! operator fuer zuweisen von Kurven
TCurve&        TCurve::operator=( const TCurve& crv )
{
  long        lSize;
  CPoint FAR*        lpp;

  //! wenn versuch Kurve sich selbst zuzuweisen dann Pointer auf Kurve
  //! zurueckgeben
  if ( this == &crv ) return * this;
  //! die Min und die Max werte kopieren + alle Parameter zuruecksetzen
  memcpy( Min, crv.Min, 3 * sizeof( float ) );
  memcpy( Max, crv.Max, 3 * sizeof( float ) );
  bEnd = FALSE;
  ActualIdx = -1;
  MaxPointIdx = -1;
  MaxMemIdx = -1;

  //! bestimmen des Speicherbedarfs der Punkte der zu uebergebenden kurve
  lSize = ( crv.MaxPointIdx + 1 ) * sizeof( CPoint );
  //! GlobalCompact hat keine Bedeutung bei Win32API
  //! wenn soviel speicher nicht verfuegbar ist dann abbruch
  if ( (DWORD)lSize > GlobalCompact( lSize ) )
			return * this;

  // Segment erweitern
  //! benoetigten Speicher fuer X,Y,Z und V reservieren
  hMemX = GlobalReAlloc( hMemX, ( crv.MaxPointIdx + 1 ) * sizeof(CPoint), 0 );
  //! wenn Fehler dann abbruch
  if ( !hMemX  )
			return * this;
  //! Kurvenindizes setzen
  MaxMemIdx = ( int ) crv.MaxPointIdx;
  ActualIdx = 0;
  MaxPointIdx = crv.MaxPointIdx;
  //! alle X-werte kopieren
  lpPX = ( CPoint* ) GlobalLock( hMemX );
  lpp = ( CPoint* ) GlobalLock( crv.hMemX );
  memcpy( lpPX, lpp, ( MaxPointIdx + 1 ) * sizeof( CPoint ) );
  GlobalUnlock( hMemX );
  GlobalUnlock( crv.hMemX );
  return *this;
};

//! anhaengen eines Punktes(tripel) x,y,z an die Kurve
//! diese Funktion verzichtet auf unbed. aufruf von FastOpen und FastClose und
//! ist damit fuer eine schnelle Abfolge von Punkthinzufuegungen geeignet
//! 1.param. X-wert (erklaer. siehe TCurve-konstr.)
//! 2.param. Y-wert (erklaer. siehe TCurve-konstr.)
//! 3.param. Z-wert (erklaer. siehe TCurve-konstr.)
//! Rueckkehrcode false bei fehler (werte wurden nicht angehaengt)
//! sonst true
BOOL TCurve::FastPAdd( float &x, float &y, float &z )
{
  //! test ob Punktanzahl der Kurve langsam die max. moegl. Punktanz. erreicht
  //! wenn ja dann Speicher erweitern(in FastOpen())
  if ( MaxPointIdx > ( MaxMemIdx - 5 ) )
			FastOpen();
  //! wenn Kurve nicht geoeffnet(d.h. nicht im speicher fix.) dann 'oeffnen'
  //! und abbruch wenn dies nicht moeglich
  if ( !bStreamOpen )
			if ( !FastOpen() )
					 return FALSE;
  //! Zaehler fuer Anzahl der akt. Kurvenpunkte + aktuellen Index auf Ende
  ++MaxPointIdx;
  bEnd = FALSE;
  ActualIdx = MaxPointIdx;
  //! Werte in Speicher schreiben + Tripel als gueltigen Punkt kennzeichnen
  lpPX[ MaxPointIdx ].Pt[0] = x;
  lpPX[ MaxPointIdx ].Pt[1] = y;
  lpPX[ MaxPointIdx ].Pt[2] = z;
  lpPX[ MaxPointIdx ].Valid = TRUE;
  //! testen ob x,y,z wert des neuen Punktes ein Min bzw. Max sind wenn ja
  //! diese als neues min. bzw. max. uebernehmen
  if ( Max[ 0 ] < lpPX[ MaxPointIdx ].Pt[0] )
			Max[ 0 ] = lpPX[ MaxPointIdx ].Pt[0];
  if ( Min[ 0 ] > lpPX[ MaxPointIdx ].Pt[0] )
			Min[ 0 ] = lpPX[ MaxPointIdx ].Pt[0];
  if ( Max[ 1 ] < lpPX[ MaxPointIdx ].Pt[1] )
			Max[ 1 ] = lpPX[ MaxPointIdx ].Pt[1];
  if ( Min[ 1 ] > lpPX[ MaxPointIdx ].Pt[1] )
			Min[ 1 ] = lpPX[ MaxPointIdx ].Pt[1];
  if ( Max[ 2 ] < lpPX[ MaxPointIdx ].Pt[2] )
			Max[ 2 ] = lpPX[ MaxPointIdx ].Pt[2];
  if ( Min[ 2 ] > lpPX[ MaxPointIdx ].Pt[2] )
			Min[ 2 ] = lpPX[ MaxPointIdx ].Pt[2];

  return TRUE;
};

//! liest ein Wertetripel aus Kurve aus (das auf das der aktuelle Index zeigt)
//! diese Funktion verzichtet auf unbed. aufruf von FastOpen und FastClose und
//! ist damit fuer eine schnelle Abfolge von Punktauslesungen geeignet
//! 1. Param. X-wert (erklaer. siehe TCurve-konstr.)
//! 2. Param. Y-wert (erklaer. siehe TCurve-konstr.)
//! 3. Param. Z-wert (erklaer. siehe TCurve-konstr.)
//! wenn das Tripel als ungueltig gekennz. wurde d.h. V ist false dann wird
//! x=y=z=0 als ergebnis geliefert ansonsten die wirkl. x,y,z werte
//! Rueckkehrcode false wenn Fehler( Kurve hat keine Punkte, Speicherfehler,..)
//! sonst true
BOOL TCurve::FastPGet( float &x, float &y, float &z )
{
  //! test ob Kurvenende schon erreicht wenn ja abbruch
  if ( bEnd )
			return FALSE;
  //! test ob Kurve ueberhaupt Punkte hat wenn nein abbruch
  if ( -1 == MaxPointIdx )
			return FALSE;
  //! test ob Kurve 'geoeffnet' wenn nein dann 'oeffnen', wenn dabei fehler
  //! dann abbruch
  if ( !bStreamOpen )
			if ( !FastOpen() )
					 return FALSE;
  //! festlegen ob Kurvenende bei naechstem aufruf erreicht
  bEnd = ( ActualIdx == MaxPointIdx );
  //! wenn der Wert als gueltig gekennzeichnet dann werte aus Speicher holen
  //! sonst x=y=z=0 zurueckliefern (fehler bei darstellung)
 if ( lpPX[ ActualIdx ].Valid )
  {
			x = lpPX[ ActualIdx ].Pt[0];
			y = lpPX[ ActualIdx ].Pt[1];
			z = lpPX[ ActualIdx ].Pt[2];
  }
  else
			x = y = z = 0.0;
  //! wenn im naechsten schritt ende der kurve noch nicht erreicht wird akt.
  //! Punktindex erhoehen
  if ( !bEnd )
			ActualIdx++;
  return TRUE;
};

//! 'oeffnet' Kurve macht damit Zugriffe (auslesen oder punkt anhaengen) erst
//! moeglich
//! sorgt dafuer, das der Speicherbereich der die X,Y,Z und V werte enthaelt
//! nicht mehr verschoben wird
//! Rueckkehrcode false bei fehler sonst true
BOOL TCurve::FastOpen( void )
{

  BOOL        bTryCompact = FALSE;
  //! test ob Kurve schon geoeffnet wenn ja schliessen
  if ( bStreamOpen )
			FastClose();
  //! test ob Punktanzahl der Kurve langsam die max. moegl. Punktanz. erreicht
  //! wenn ja dann Speicher um GrowUnit(=50) erweitern
  if ( MaxPointIdx > ( MaxMemIdx - 5 ) )
  {
			MaxMemIdx += GrowUnit;
			while ( 1 )
			{
					 // Segment erweitern
					 hMemX = GlobalReAlloc( hMemX, ( MaxMemIdx + 1 ) * sizeof(CPoint), 0 );
					 //! wenn erweiterung erfolgreich dann alle diesen neuen Punkte als un-
					 //! gueltig kennzeichnen vorher v-speicherbereich fixieren
					 lpPX = ( CPoint* ) GlobalLock( hMemX );//*
					 if ( hMemX )
					 {
						for( int zae=MaxPointIdx+1;zae < MaxMemIdx;zae++)
						 lpPX[zae].Valid = FALSE;
						goto Open;
					 }
					 //! wenn nicht erfolgreich dann alle Kurvenindezes auf init.werte
					 //! zuruecksetzen + abbrechen
					 if ( bTryCompact )
					 {
						ActualIdx = -1;
						MaxPointIdx = -1;
						MaxMemIdx = -1;
						return FALSE;
					 }
					 //! GlobalCompact hat keine Bedeutung bei Win32API
					 GlobalCompact( MaxMemIdx * ( 3 * sizeof( float ) + sizeof( BOOL ) ) );
					 bTryCompact = TRUE;
			}
  }
  //! speicherbereiche fixieren
Open:
  lpPX = ( CPoint* ) GlobalLock( hMemX );
  //! wenn nicht fixierbar dann abbruch ansonsten Kurve als offen kennzeichnen
  if ( !lpPX )
  {
			ActualIdx = -1;
			MaxPointIdx = -1;
			return FALSE;
  }
  bStreamOpen = TRUE;
  return TRUE;
};


//! 'schliesst' Kurve und macht damit zugriffe(lesende oder schreibende) auf
//! die Kurve unmoeglich
//! Rueckkehrcode immer true
BOOL TCurve::FastClose( void )
{
  //! verschieben der speicherbereiche zulassen + kurve als geschlossen
  //! kennzeichnen
  GlobalUnlock( hMemX );
  bStreamOpen = FALSE;
  return TRUE;
};


//! haengt ein Punkttripel an die Kurve an wobei am anfang die Kurve geoeffnet
//! und am ende die Kurve wieder geschlossen wird; sollte benutzt werden
//! wenn naechstes Anhaengen eines Punktes an die Kurve nicht unmittelbar folgt
//! 1.Param = X-wert (erklaer. siehe TCurve-konstr.)
//! 2.Param = Y-wert (erklaer. siehe TCurve-konstr.)
//! 3.Param = Z-wert (erklaer. siehe TCurve-konstr.)
//! Rueckkehrcode false wenn Fehler( fehler bei oeffnen der kurve) sonst true
BOOL TCurve::PAdd( float &x, float &y, float &z )
{
  //! test ob sich kurve oeffnen laesst, wenn nicht dann abbruch
  if ( !FastOpen() ) return FALSE;
  //! zaehler fuer anzahl der abgelegten punkte der kurve um 1 erhoehen + Kurve
  //! als nicht beendet kennzeichnen und akt.Index auf diesen neuen Pkt. setzen
  ++MaxPointIdx;
  bEnd = FALSE;
  ActualIdx = MaxPointIdx;
  //! x,y,z werte in speicher und dieses tripel als gueltig kennzeichnen
  lpPX[ MaxPointIdx ].Pt[0] = x;
  lpPX[ MaxPointIdx ].Pt[1] = y;
  lpPX[ MaxPointIdx ].Pt[2] = z;
  lpPX[ MaxPointIdx ].Valid = TRUE;
  //! testen ob x,y,z wert des neuen Punktes ein Min bzw. Max sind wenn ja
  //! diese als neues min. bzw. max. uebernehmen
  if ( Max[ 0 ] < lpPX[ MaxPointIdx ].Pt[0] )
			Max[ 0 ] = lpPX[ MaxPointIdx ].Pt[0];
  if ( Min[ 0 ] > lpPX[ MaxPointIdx ].Pt[0] )
			Min[ 0 ] = lpPX[ MaxPointIdx ].Pt[0];
  if ( Max[ 1 ] < lpPX[ MaxPointIdx ].Pt[1] )
			Max[ 1 ] = lpPX[ MaxPointIdx ].Pt[1];
  if ( Min[ 1 ] > lpPX[ MaxPointIdx ].Pt[1] )
			Min[ 1 ] = lpPX[ MaxPointIdx ].Pt[1];
  if ( Max[ 2 ] < lpPX[ MaxPointIdx ].Pt[2] )
			Max[ 2 ] = lpPX[ MaxPointIdx ].Pt[2];
  if ( Min[ 2 ] > lpPX[ MaxPointIdx ].Pt[2] )
			Min[ 2 ] = lpPX[ MaxPointIdx ].Pt[2];

  //! kurve schliessen
  FastClose();
  return TRUE;
};

//! liest ein wertetripel aus der Kurve aus ; diese Funktion prueft, ob die
//! Kurve bei aufruf geschlossen war und schliesst sie dann am ende wieder
//! (ansonsten verhaelt sie sich wie FastPGet())
//! 1. Param. X-wert (erklaer. siehe TCurve-konstr.)
//! 2. Param. Y-wert (erklaer. siehe TCurve-konstr.)
//! 3. Param. Z-wert (erklaer. siehe TCurve-konstr.)
//! wenn das Tripel als ungueltig gekennz. wurde d.h. V ist false dann wird
//! x=y=z=0 als ergebnis geliefert ansonsten die wirkl. x,y,z werte
//! Rueckkehrcode false wenn Fehler( Kurve hat keine Punkte, Speicherfehler,..)
//! sonst true
BOOL TCurve::PGet( float &x, float &y, float &z )
{
  //! merkt sich ob kurve bei aufruf geoeffnet war
  BOOL        bWasOpen = bStreamOpen;
  //! test ob Kurvenende schon erreicht wenn ja abbruch
  if ( bEnd )
			return FALSE;
  //! test ob Kurve ueberhaupt Punkte hat wenn nein abbruch
  if ( -1 == MaxPointIdx )
			return FALSE;
  //! test ob Kurve 'geoeffnet' wenn nein dann 'oeffnen', wenn dabei fehler,
  //! dann abbruch
  if ( !bStreamOpen )
			if ( !FastOpen() )
					 return FALSE;
  //! festlegen ob Kurvenende bei naechstem aufruf erreicht
  bEnd = ( ActualIdx == MaxPointIdx );
  //! wenn der Wert als gueltig gekennzeichnet dann werte aus Speicher holen
  //! sonst x=y=z=0 zurueckliefern (fehler bei darstellung)
  if ( lpPX[ ActualIdx ].Valid )
  {
			x = lpPX[ ActualIdx ].Pt[0];
			y = lpPX[ ActualIdx ].Pt[1];
			z = lpPX[ ActualIdx ].Pt[2];
  }
  else
			x = y = z = 0.0;
  //! wenn im naechsten schritt ende der kurve noch nicht erreicht wird akt.
  //! Punktindex erhoehen
  if ( !bEnd )
			ActualIdx++;
  //! kurve wieder schliessen wenn sie am anfang geoeffnet war
  if ( !bWasOpen )
			FastClose();
  return TRUE;
};


//! liefert den in der Kurve vorhandenen dem Param 'search' naehsten X-wert(den
//! indexwert) als Rueckkehrcode zurueck und den zugehoerigen Y-Wert sowie den
//! abstand zwischen X-wert und search
//! 1.param (eingabe) search in Kurve soll nach einem X-punkt gesucht werden,
//!                   der zu search den geringsten abstand besitzt
//! 2.Param (ausgabe) delta enthaelt nach aufruf den Abstand von search zum
//!                   naehsten X-wert
//! 3.Param (ausgabe) value enthaelt nach aufruf den zum naehsten X-wert
//!                   gehoerigen Y-wert(Intensitaet)
//! Rueckkehrcode ist der Index fuer den gilt X[Index] ist naehster X-wert zu
//!                   search, wenn search im inneren des x-bereichs der Kurve
//!                   ist, sonst (wenn ausserhalb dieses bereiches oder bei
//!                   fehler) den Wert 0
//! voraussetzung ist, das die X-werte der kurve von klein nach gross sortiert
//!                   sind
int        TCurve::GetValueByValue( float search, float&delta, float&value )
{
  int        Sign, Idx, BB;

  // the X column should be sorted
  delta = value = 0.0;
  //! test ob search im x-bereich + ob Kurve ueberhaupt punkte hat + ob Kurve
  //! sich oeffnen laesst    wenn nein abbruch mit 0
  if ( ( search > Max[ 0 ] ) || ( search < Min[ 0 ] ) )
			return 0;
  if ( -1 == MaxPointIdx )
			return 0;
  if ( !FastOpen() )
			return 0;
  //! binaere suche in X-werten der kurve
  Idx = BB = MaxPointIdx / 2;
  while ( BB )
  {
			BB /= 2;
			Sign = ( search > lpPX[ Idx ].Pt[0] ) ? 0 : 1;//*
			if ( Sign )
					 Idx -= BB;
			else
					 Idx += BB;
  };
  //! wenn search zwischen zwei x-werten dann naeheren nehmen
  if ( Sign )
			if ( ( search - lpPX[ Idx ].Pt[0] ) > ( lpPX[ Idx + 1 ].Pt[0] - search ) )//*
					 Idx++;
			else
					 if ( ( search - lpPX[ Idx - 1 ].Pt[0] ) < ( lpPX[ Idx ].Pt[0] - search ) )
						Idx--;
  //! ausgabeparameter belegen + kurve schliessen
  value = lpPX[Idx].Pt[1];
  delta = search - lpPX[ Idx ].Pt[0];//*
  FastClose();
  return Idx;
};


//! setzt den internen Index des gerade aktuellen Tripels auf den angegebenen
//! wert wenn dies moeglich ist
//! 1.param pos ist der wert auf den der Index gesetzt werden soll; wenn pos
//!         kleiner als 0 ist wird der Index auf 0 gesetzt; wenn pos groesser
//!         als der index des letzten Punktes der Kurve ist wird der der Index
//!         auf diesen letzten Punkt gesetzt
//! Rueckkehrcode false wenn Kurve keine Punkte hat sonst true
BOOL TCurve::SetPP( int pos )
{
  //! test ob Kurve punkte hat wenn nicht abbruch
  if ( MaxMemIdx == -1 )
			return FALSE;
  //! setzen des Indizes auf Wert(mit test ob groesser bzw. kleiner als erlaubt
  //! und setzen des kleinsten bzw. groessten erlaubten wertes wenn dem so ist)
  //!  + kurve als nicht fertig kennzeichnen
  ActualIdx = ( pos > MaxPointIdx ) ? MaxPointIdx : pos;
  ActualIdx = ( ActualIdx < 0 ) ? 0 : ActualIdx;
  bEnd = FALSE;
  return TRUE;
};


//! addiert den wert val zum Y-wert des Tripels x,y,z auf das der aktuelle
//! Index zeigt
//! 1.param val wird auf Y-wert aufaddiert
//! Rueckkehrcode false wenn Fehler sonst true
BOOL TCurve::ValueAdd( float &val )
{
  // Addiert val jeweils zu [1] des aktuellen Punktes
  //! test ob kurve punkte hat wenn nicht abbruch
  if ( MaxPointIdx == -1 )
			return FALSE;
  //! test ob am Kurven ende wenn ja dann abbruch
  if ( bEnd )
			return FALSE;
  //! setzen ob im naechsten aufruf kurvenende erreicht
  bEnd = ( ActualIdx == MaxPointIdx );
  //! fixieren des speicherbereichs der y-werte (verschieben verhindern)
  //! und wenn dies nicht moeglich kurvenparam. auf init. werte zuruecksetzen +
  //! abbruch
  lpPX = ( CPoint* ) GlobalLock( hMemX );
  if ( !lpPX )
  {
			ActualIdx = -1;
			MaxPointIdx = -1;
			return FALSE;
  }
  //! aufaddieren von val auf den akt.y-wert
  lpPX[ ActualIdx ].Pt[1] += val;
  //! wenn noch nicht am ende der kurve angekommen dann akt.Index erhoehen
  if ( !bEnd )
			ActualIdx++;
  //! verschieben von y-werte speicherbereich wieder zulassen
  GlobalUnlock( hMemX );
  return TRUE;
};


//! legt alle x,y,z werte einer Kurve in einer Datei ab
//! 1.Param filename ist der name der Datei in der die Kurve abgelegt werden
//! soll
//! Rueckkehrcode false wenn Fehler(Datei laesst sich nicht anlegen) sonst true
BOOL TCurve::Save( LPCSTR filename )
{
  OFSTRUCT of;
  int hFile;
  float x, y, z;
  char buf[ MaxString ];

  //!  erstellen einer Datei mit angeg. namen
  hFile = OpenFile( filename, &of, OF_CREATE );
  //! wenn fehler dann abbruch
  if ( -1 == hFile ) return FALSE;
  //! internen Index auf 1.Tripel der Kurve setzen + filepointer auf ende der
  //! Datei + Kurve oeffnen
  SetPP( 0 );
  _llseek( hFile, 0, SEEK_END );
  FastOpen();
  //! ein tripel aus Kurve holen(incl. index erhoehen) solange bis fehler(ende
  //! erreicht)
  while ( FastPGet( x, y, z ) )
  {
			//! ein tripel (x,y,z) als einer Zeile in datei schreiben
			sprintf( buf, "%.3f %.3f %.3f\n", x, y, z );
			_lwrite( hFile, ( LPSTR ) buf, strlen( buf ) );
  }
  //! datei schliessen + kurve schliessen
  _lclose( hFile );
  FastClose();
  return TRUE;
};


//! speichert Kurve in angegebener Datei unter angegebener Option
//! 1.Param filename ist der name der Datei in der die Kurve abgelegt werden
//!         soll
//! 2.Param type soll mehrere moeglichkeiten der art und weise wie und was von
//!         der der Kurve abgelegt werden soll spezifizieren
//!         (bisher nur type=100 implementiert=>legt nur die Y-werte der kurve
//!         in der Datei ab)
//! Rueckkehrcode false wenn Fehler(Datei laesst sich nicht anlegen) sonst true
BOOL TCurve::Save( LPCSTR filename, int type, int )
{
  OFSTRUCT of;
  int hFile;
  float x, y, z;
  char buf[ MaxString ];


  //!  erstellen einer Datei mit angeg. namen
  hFile = OpenFile( filename, &of, OF_CREATE );
  if ( -1 == hFile )
			return FALSE;
  //! internen Index auf 1.Tripel der Kurve setzen + filepointer auf ende der
  //! Datei + Kurve oeffnen
  SetPP( 0 );
  _llseek( hFile, 0, SEEK_END );
  FastOpen();
  //! abhaengig von gewaehlter Option (hier nur eine implem.)
  switch ( type )
  {
			case 100:
					 //! ein tripel aus Kurve holen(incl. index erhoehen) solange bis fehler
					 //! (Ende erreicht)
					 while ( FastPGet( x, y, z ) )
					 {
						//! einen Y-wert pro zeile in datei schreiben
						sprintf( buf, "%.5f\n", y );
						_lwrite( hFile, ( LPSTR ) buf, strlen( buf ) );
					 }
					 break;
  }
  //! datei schliessen + kurve schliessen
  _lclose( hFile );
  FastClose();
  return TRUE;
};


//! berechnet eine durch den letzen Param. bestimmte Fkt
//! (max. Intensitaet/Schwerpunkt)
//! 1.Param(ausgabe) x enthaelt nach aufruf je nach funktion berechneten Wert
//! 2.Param(ausgabe) y enthaelt nach aufruf je nach funktion berechneten Wert
//! 3.Param(ausgabe) z enthaelt nach aufruf je nach funktion berechneten Wert
//! 4.Param(eingabe) func ermoeglicht verschieden funktionen aufzurufen (nur 2
//!                  implementiert)
//!                  func=2  dann x,y,z tripel ermitteln, fuer das y (die
//!                          Intensitaet) maximal ist
//!                  func<>2 dann x,y,z Schwerpunkt der Kurve ermitteln
//!                          (wird scheinbar nicht benutzt)
//! Rueckkehrcode false wenn fehler sonst true
BOOL TCurve::GetGravityCenter( float &x, float &y, float &z, int func )
{
  int idx, maxpos;
  //! nicht von diffr.gruppe auskommentiert
  //! float     retval = 0.0;
  double integral, weigth, dx;

  //! wenn zu wenig punkte in Kurve (1 oder keiner) dann abbruch
  if ( 2 > MaxPointIdx )
			return FALSE;
  // berechnet den Schwerpunkt der Kurve und gibt den entsprechenden relativen
  // Winkel zurueck
  //! fixieren der Speicherbereiche (verhindert verschieben waehrend zugriff)
  lpPX = (CPoint* ) GlobalLock( hMemX );
  //! wenn nicht moeglich dann Kurvenparam. auf init.werte zuruecksetzen
  if ( !lpPX )
  {
			ActualIdx = -1;
			MaxPointIdx = -1;
			return FALSE;
  }
  //! abhaengig von ausgewaehlter Fkt. berechnen
  switch ( func )
  {
					 //! tripel max. y-wertes(intensitaet) bestimmen (fehler)
			case 2:
					 //! sequentielle suche nach groesstem y wert
					 maxpos = 0;
					 for ( idx = 0; idx <= MaxPointIdx; idx++ )
						if ( lpPX[ idx ].Pt[1] > lpPX[ maxpos ].Pt[1] )
									 maxpos = idx;
					 //! uebergeben des max. y-werte und der zugehoerigen x,z werte
					 x = lpPX[ maxpos ].Pt[0];
					 y = lpPX[ maxpos ].Pt[1];
					 z = lpPX[ maxpos ].Pt[2];

					 break;

					 //! ? berechnet Schwerpunkt
			default:
					 integral = weigth = 0.0;
					 dx = lpPX[ 1 ].Pt[0] - lpPX[ 0 ].Pt[0];
					 for ( idx = 0; idx <= MaxPointIdx; idx++ )
					 {
						weigth += lpPX[ idx ].Pt[1] * dx;
						integral += lpPX[ idx ].Pt[1] * ( idx * dx ) * dx;
					 }
					 if ( weigth )
						x = ( integral / weigth );

					 for ( idx = 0; idx <= MaxPointIdx; idx++ )
						if ( lpPX[ idx ].Pt[0] >= x )
						{
								 y = lpPX[ idx ].Pt[1];
								 z = lpPX[ idx ].Pt[2];
								 break;
						}
					 break;
  }
  //! speicherbereichsverschiebung wieder zulassen
  GlobalUnlock( hMemX );
  return TRUE;
};


//! markiert alle Tripel der Kurve als ungueltig, deren y-wert(Intensitaet)
//! kleiner als das Level-fache des Intensitaetsbereichs(max.Y- min.Y) der
//! Kurve ist
//! 1.Param Level  alle Y-werte die kleinergleich Level* Intensitaetsbereich
//!                sind (und die zugehoer.x,z werte)werden als ungueltig
//!                gekennzeichnet (nur Werte 0<=Level<=1 sind sinnvoll)
//! Rueckkehrcode false wenn Fehler sonst true
BOOL TCurve::DeleteUnderGround( float Level )
{
  int Idx = 0;
  //! schwellwert den Y ueberschreiten muss um gueltig zu sein festlegen
  double PeakMin = ( Max[ 1 ] - Min[ 1 ] ) * Level;

  // Trennen der Peaks vom Untergrund
  // Es wird nur y bearbeitet
  // Die gueltigen Punkte sind noch durch ungueltige getrennt
  //! test ob Kurve Punkte hat wenn nicht dann abbruch
  if ( -1 == MaxPointIdx )
			return FALSE;
  //! speicherbereiche fixieren
  lpPX = ( CPoint* ) GlobalLock( hMemX );
  //! wenn nicht moeglich dann abbruch
  if ( !lpPX )
			return FALSE;
  //! kurve durchlaufen + alle tripel als ungueltig kennzeichnet deren y-werte
  //! den schwellwert nicht ueberschreiten
  while ( Idx <= MaxPointIdx )
  {
			if ( ( lpPX[ Idx ].Pt[1] - Min[ 1 ] ) <= PeakMin )
					 lpPX[ Idx ].Valid = FALSE;
			Idx++;
  };
  //! verschieben der speicherbereiche wieder zulassen
  GlobalUnlock( hMemX );
  return TRUE;
};


//!  wird nicht benutzt
BOOL TCurve::DeleteFlanks()
{
  int MaxValIdx;
  int Idx = 0;

  // Flanken der Peaks beseitigen
  // Es wird nur y bearbeitet
  // Die gueltigen Punkte werden nach vorn geschoben + der Rest wird geloescht
  if ( -1 == MaxPointIdx )
			return FALSE;
  lpPX = ( CPoint* ) GlobalLock( hMemX );
  if ( !lpPX  )
			return FALSE;
  while ( Idx <= MaxPointIdx )
  {
			// Bis zum naechsten Peak
			while ( !lpPX[ Idx ].Valid )
					 if ( Idx < MaxPointIdx )
						Idx++;
					 else break;   // Ende erreicht
			// Groessten Wert suchen
			MaxValIdx = Idx;
			if ( Idx < MaxPointIdx )
					 Idx++;
			else break;   // Ende erreicht
			while ( lpPX[ Idx ].Valid )
			{
					 if ( lpPX[ Idx ].Pt[1] > lpPX[ MaxValIdx ].Pt[1] )
					 {
						lpPX[ MaxValIdx ].Valid = FALSE;
						MaxValIdx = Idx;
					 }
					 else
						lpPX[ Idx ].Valid = FALSE;
					 if ( Idx < MaxPointIdx )
						Idx++;
					 else break;   // Ende erreicht
			}
			// Peak nach vorn kopieren

			//lpPnts[Idx] = lpPoints[MaxValIdx];

			lpPX[ Idx ].Valid = TRUE;
			lpPX[ MaxValIdx ].Valid = FALSE;
			if ( Idx < MaxPointIdx )
					 Idx++;
  }
  // es wurden peaks gefunden
  //! nicht von diffr.gruppe auskommentiert
  //! Exit:
  GlobalUnlock( hMemX );
  return TRUE;
};


//! nicht von diffr.gruppe auskommentiert
//! BOOL TCurve::GetReflection(int /*order*/,LPCPoint /*Peak*/)
/*
		  {
		  //! static BOOL   bSorted = FALSE;    // aendern
		  //! BOOL          bChange = FALSE; //wird nur im auskomm. Teil benutzt
		  BOOL          Ok = FALSE;
		  // int           Idx, MaxValIdx;
		  // CPoint        merker;

		  // Alle Peak-Maxima werden der Groesse nach sortiert
		  // Der groesste Peak hat die Ordnung 0 usw.
		  // Es sind mit weniger als 20 Peaks zu rechnen
		  if(-1 == MaxPointIdx)
					 return FALSE;
		  lpPX = (float*)GlobalLock(hMemX);
		  lpPY = (float*)GlobalLock(hMemY);
		  lpPZ = (float*)GlobalLock(hMemZ);
		  lpPV = (BOOL*)GlobalLock(hMemV);
		  if(!lpPX || !lpPY || !lpPZ || !lpPV)
					 goto Exit;
//Kommentarbeginn
		  if(!lpPV[0])
					 goto Exit;
		  if(0 < MaxPointIdx)
					 Idx = 1;
					 else
					 {
					 bSorted = TRUE;
					 //*Peak = lpPoints[0];
					 if(order == 0)
								Ok = TRUE;
								else
								Ok = FALSE;
					 goto Exit;
					 }
		  if(!bSorted)
					 {
					 // Sortieren  (Bubblesort)
					 Idx = 0;
					 // naechsten groessten Wert suchen
					 while(Idx < MaxPointIdx)
								{
								while(Idx <= MaxPointIdx)
										  {
										  if(lpPY[Idx] > lpPY[ActualIdx])
													 {
													 MaxValIdx = Idx;
													 bChange = TRUE;
													 }
										  Idx++;
										  }
								if(bChange)
										  {
										  // Mit ersten Punkt austauschen
										  merker = lpPoints[ActualIdx];
										  lpPoints[ActualIdx] = lpPoints[MaxValIdx];
										  lpPoints[MaxValIdx] = merker;
										  bChange = FALSE;
										  // point geht weiter als ThisPoint
										  if(ActualIdx < MaxPointIdx)
													 ActualIdx++;
										  Idx = ActualIdx+1;
										  }
								}
					 bSorted = TRUE;
					 }
		  ActualIdx = 0;
		  while(order--)
					 if(ActualIdx < MaxPointIdx)
								ActualIdx++;
								else
								// Peak der angegebenen Ordnung ist nicht vorhanden
								Ok = FALSE;
		  // Werte kopieren
		  (*Peak) = lpPoints[ActualIdx];
//Kommentarende
Exit:
		  GlobalUnlock(hMemX);
		  GlobalUnlock(hMemY);
		  GlobalUnlock(hMemZ);
		  GlobalUnlock(hMemV);
		  if(Ok)
					 return TRUE;
					 else
					 return FALSE;
		  };
*/


//! liefert die Breite des 1.Bereiches, in dem alle y-werte des Tripels die
//! Haelfte des Intensitaetbereichs(Y(max)-Y(min)) uebersteigen, sowie aus
//! diesem Bereich x-wert,y-wert des Tripels fuer das Y-wert(Intensitaet)
//! maximal ist
//! 1. bereich bedeutet der am weitesten links liegende Bereich der obige
//! Eigenschaften erfuellt
//! 1.Param(ausgabe) hw enthaelt nach aufruf breite des Bereichs
//! 2.Param(ausgabe) pos enthaelt nach aufruf X-wert an der max. y-wert
//!                      (Intensitaet) in diesem Bereich
//! 3.Param(ausgabe) intens enthaelt nach aufruf max.y-wert(intensitaet) in
//!                         diesem Bereich
//! Rueckkehrcode false wenn fehler sonst true
//! anmerkung: dieser Bereich darf nicht bis ans ende der Kurve reichen
//! d.h. kurve darf nicht monoton wachsend sein (sonst return false)
BOOL TCurve::GetPeakProperties( float &hw, float &pos, float &intens )
{
  int LeftPIdx, RightPIdx, MaxPIdx, Idx;
  BOOL retval = FALSE;

  //! test ob kurve punkte hat wenn nicht abbruch
  if ( -1 == MaxPointIdx )
			return FALSE;
  //! ermitteln der Bereiche, die obige bedingung erfuellen und bei fehler
  //! Abbruch
  if ( !DeleteUnderGround( 0.5 ) ) return FALSE;
  // 000001111111100000
  // Bis zum naechsten Peak
  Idx = 0;
  //! speicherbereiche fixieren + test ob erfolgreich wenn nicht dann abbruch
  lpPX = ( CPoint* ) GlobalLock( hMemX );
  if ( !lpPX )
			goto Exit;
  //! ueberspringen eines event. vorhandenen ungueltigen Bereichs gleich am
  //! anfang der kurve + abbruch falls die gesamte kurve ungueltig ist
  while ( !lpPX[ Idx ].Valid )
			if ( Idx < MaxPointIdx )
					 Idx++;
			else
					 goto Exit;  // Ende erreicht
  // linke Flanke
  LeftPIdx = MaxPIdx = Idx;
  //! ermitteln des max. y-wertes in diesem Bereich + abbruch falls auf diesen
  //! gueltigen bereich nicht ein ungueltiger folgt
  while ( lpPX[ Idx ].Valid )
  {
			if ( lpPX[ Idx ].Pt[1] > lpPX[ MaxPIdx ].Pt[1] )
					 MaxPIdx = Idx;
			if ( Idx < MaxPointIdx )
					 Idx++;
			else
					 goto Exit;  // Ende erreicht
  }
  // rechte Flanke

  RightPIdx = Idx - 1;
  //! berechnen und uebergeben der ausgabeparameter
  hw = fabs( lpPX[ RightPIdx ].Pt[0] - lpPX[ LeftPIdx ].Pt[0] );
  pos = lpPX[ MaxPIdx ].Pt[0];
  intens = lpPX[ MaxPIdx ].Pt[1];
  //! funktion erfolgreich
  retval = TRUE;
Exit:
  //! verschieben der speicherbereiche wieder zulassen
  GlobalUnlock( hMemX );
  return retval;
};
// __LastLine__



