#ifndef _M_DLGDIF_H_
#define _M_DLGDIF_H_


class TCalibratePsd : public TModalDlg
{
public:
    TCalibratePsd( TCurve& );

	 //! ermittelt Anzahl der Scrollbarschritte links bzw. rechts von der
    //! absoluten Null
    //! 1. Parameter side==LEFT Anzahl der Scrollbarschritte links
	 //! von der abs. Null
    //! sonst Anzahl der Scrollbarschritte rechts
    //! von der abs. Null
    //! Rueckkehrcode Anzahl der Scrollbarschritte links bzw. rechts
    //! von der abs. Null
    //! Achtung diese Funktion muss mit LEFT zuerst aufgerufen werden
    int GetBarEgde( int );

    //! liefert die Position des Scrollbar anhand der akt. Motorposition im
    //! Rueckkehrcode
    int GetBarPos( void );

    //! behandelt alle Scrollbarereignisse
    void Dlg_OnHScrollBar( HWND, HWND, UINT, int );

	 //! setzt Defaultwerte der Dialogbox bei Aufruf der Box
	 //! Rueckkehrcode immer true
	 BOOL Dlg_OnInit( HWND, HWND, LPARAM );

	 //! behandelt Timersignale zur aktualisierung der Winkelwerte in Dialogbox
	 void Dlg_OnTimer( HWND, UINT );

	 //! behandelt Kommandos in der Dialogbox
	 void Dlg_OnCommand( HWND, int, HWND, UINT );

	 //! wird aufgerufen beim druecken des Beenden-Buttons
	 //! Rueckkehrcode False wenn der Winkel pro Kanal ==0
	 //! sonst true
	 BOOL CanClose( void );

    //! wird bei verlassen der dialogbox aufgerufen Rueckkehrcode immer true
    BOOL LeaveDialog( void );

private:

    //! Zeitintervall zur akt. der Winkel in Dialogbox
    int nAskTime;

    //! gibt an ob motor in bewegung
    BOOL bMoveActive;

    //! gibt umrechnungsfactor an um Winkelsekunden in die jeweilige Einheit
    //! umzurechnen
	 float fRelationFactor;

	 //! wahrscheinlich ueberfluessig
	 BOOL bCalibrationOk;

	 //! akt. angezeigte Kurve
    LPCurve Curve;

    //! gibt pos. des linken bzw. rechten Peaks an
    CPoint LeftPeak;
	 CPoint RightPeak;

    //! gibt an ob rechter bzw. linker Peak schon erreicht ist
    BOOL bGetRightPeak;
    BOOL bGetLeftPeak;

    //! gibt an ob motor ueber groessere distanz faehrt
    BOOL bLongMove;

    //! scheinbar ueberfluessig
	 float fOldPsdStep;

    //! FIX Fehler 49
    //! beinhaltet die Schrittweite vor Aufruf der Calibr.-Box
    double dOldMotorWidth;

    //! Handle auf Dialogelement
    HWND hDlgItem;

    //! Handle des Scrollbars
	 HWND hBar;

    //! gibt an ob Abzahl der Scrollbarschritte kleiner als 20000 ist (1) oder
	 //! nicht (2)
    int BarFactor;

    //! enthaelt Nummer des Thetamotors in der Motorenliste
    int nMotor;

    //! FIX Fehler 48
    //! Wert fuer Winkel pro Kanal
    float dStepWidth;

    //! Pointer auf einen Psd
    TDevice* Device;
};

//*****************************************************************************
class TChooseScan : public TModelessDlg
{
public:
	 TChooseScan( TAreaScan* );

private:

	 //! setzt Defaultwerte der Dialogbox bei Aufruf der Box
	 //! Rueckkehrcode immer true
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );

    //! behandelt Kommandos in der Dialogbox
    void Dlg_OnCommand( HWND, int, HWND, UINT );

    //! wird aufgerufen, wenn Scrollbar angeclickt wurde
    void Dlg_OnVScrollBar( HWND, HWND, UINT, int );

    //! wird aufgerufen, wenn in der Dialogbox die linke Maustaste nach
    //! Druecken wieder losgelassen wird (sinnvoll wenn Dialogbox waehrend
    //! eines Areascans aktiv ist)
    void Dlg_OnLButtonUp( HWND, int, int, UINT );

	 //! wird aufgerufen beim druecken des OK-Buttons
    //! Rueckkehrcode immer true
    BOOL CanClose( void );

    //! Pointer auf das zugehoerige Areascanfenster
    TAreaScan* AScan;

    //! Pointer auf akt. angezeigte Kurve
    TCurve* Curve;

    //! Pointer auf Scanreportkurve der datenbasis
    TCurve* Report;

    //! Handle des Scrollbars
    HWND BarHandle;

    //! Handle der Info-Combobox
    HWND hInfoList;

    //! enthaelt Nr. des letzten Scans in Datenbasis
	 int nMaxScan;

	 //! beinhaltet zuordnung (Nr. des Comboxeintrages zu zusatzinfo)
	 int InfoField[5];

   //! Pointer auf 2.Scanreportkurve (zusatzinfos)
   TCurve* Report2;

   //! enthaelt Nr. der aktuell ausgewaehlten zusatzinfo
   //! 0-Messdetektor / 1-integr.Int. / 2-absorber / 3-Monitorint / 4-messzeit 
   int nInfoNumber;
};


class TSetupAreaScan : public TModalDlg
{
   //! dieser friend-Dialog wird nur aus diesem Dialog aufgerufen
	 friend class TSetOffset;

public:
    TSetupAreaScan( TAreaScan* );

private:

	 //! Formatstrings fuer die Motorwerte
	 char szOWF[ 7 ], szTWF[ 7 ];

	 //! enthaelt Nummer des Omega- sowie Thetamotors in der Motorenliste
    int nOmega, nTheta,

    //! scheinbar ueberfluessig
    nOldMotor;

    //! gibt ausgewaehlten Detektor an
	 LPDevice Device;

   //! identifiziert Monitordetektor
 	 LPDevice Monitor;

    //! Pointer auf AreaScanfenster
	 TAreaScan* Scan;

    //! Handle der Detektorauswahlbox
    HWND hDeviceList;

    //! realisiert test der Dialogboxinhalte auf ihre konsistenz vor verlassen
    //! der Dialogbox (durch OK-Button)
    //! Rueckkehrcode false wenn die Dialogbox nicht geschlossen werden kann
    //! (Werte inkonsistent) sonst true
    BOOL CanClose( void );

    //! behandelt Kommandos in der Dialogbox
    void Dlg_OnCommand( HWND, int, HWND, UINT );

    //! setzt Defaultwerte der Dialogbox bei Aufruf der Box
    //! Rueckkehrcode immer true
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );

    //! wird bei verlassen der Dlg.box aufgerufen (in dieser Dlg.box keine
    //! Bedeutung) da auch geerbt
    BOOL LeaveDialog( void );

    //! beinhaltet den Wert des Feldes Minumum im Gruppenfeld 2Theta...
    double d2ThetaStart;

    //! beinhaltet o-t-offset
    double dOmegaOffset;
    double dThetaOffset;

};


//! diese Dialogbox funktioniert nicht(unvollstaendig)
class TPsdRemoteSync : public TModalDlg
{
public:
	 TPsdRemoteSync( TCurve& );
	 BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	 void Dlg_OnTimer( HWND, UINT );
	 void Dlg_OnCommand( HWND, int, HWND, UINT );
	 BOOL CanClose( void );
	 BOOL LeaveDialog( void );

private:
	 void PrintFileName( void );
	 LPCSTR GetFileName( void );
	 int nAskTime;
	 BOOL bCalibrationOk;
	 LPCurve Curve;
	 float fOldPsdStep;
	 HWND hDlgItem;
	 TDevice* Device;
	 int nCurveCount;
	 char aPath[ 50 ];
	 char aName[ 14 ];
	 float fExposureTime;
	 DWORD dwExposureCounts;
	 float fError;
	 char aFileName[ 127 ];
};


//! Dialog zur eingabe des Omega- und Thetaoffset
class TSetOffset : public TModalDlg
{
public:
	 TSetOffset( BOOL, TSetupAreaScan* );

private:
  //! gibt an ob die Thetaachse im AreaScan-dialog fixiert war;
  //! in diesem Fall soll nur Omegaoffset eingebbar sein
	BOOL bOnlyOmega;

  //! Wert OmegaOffset
	double dOmegaOffset;

  //! Wert ThetaOffset
	double dThetaOffset;

  //! Wert Wunschwinkel(omega)
	double dIdealOmega;

  //! Wert Wunschwinkel(theta)
	double dIdealTheta;

  //! gibt an ob direkt Offset eingegeben wurde(True) oder oder
  //! ueber Winkelwerte berechnung erfolgen muss(False) Omega
	BOOL bOmegaDirect;

  //! gibt an ob direkt Offset eingegeben wurde(True) oder oder
  //! ueber Winkelwerte berechnung erfolgen muss(False) Theta
	BOOL bThetaDirect;

  //! pointer auf AreaScandialog
	TSetupAreaScan* dialog;

  //! schliesst dialog
  BOOL CanClose( void );

  //! behandelt dialogereignisse
  void Dlg_OnCommand( HWND, int, HWND, UINT );

  //! Initialisierung der Dialogwert
  BOOL Dlg_OnInit( HWND, HWND, LPARAM );

};

//! Aenderung DB_Zerlegung
//! Dialogbox-Klasse fuer Einstellungen Datenbasis zusammensetzen
class TComposeDB : public TModalDlg
{
public:
	 TComposeDB( TAreaScan* );

private:
	 //! zugehoeriges AreaScanfenster
	 TAreaScan* Areascan;

	 //! realisiert test der Dialogboxinhalte auf ihre konsistenz vor verlassen
	 //! der Dialogbox (durch OK-Button)
	 //! Rueckkehrcode False wenn die Dlg.box nicht geschlossen werden kann
	 //! (Werte inkonsistent) sonst true
	 BOOL CanClose( void );

	 //! behandelt alle kommandos der Dialogbox
	 void Dlg_OnCommand( HWND, int, HWND, UINT );

	 //! setzt Defaultwerte der Dialogbox bei Aufruf der Box
	 //! Rueckkehrcode immer true
	 BOOL Dlg_OnInit( HWND, HWND, LPARAM );

	 int oldMaxFiles;
	 char oldFirstFile[ MaxString ], oldLastFile[ MaxString ];
};

//! Aenderung DB_Zerlegung
//! Dialogbox-Klasse fuer Einstellungen Datenbasis zerlegen
class TDismantleDB : public TModalDlg
{
public:
	 TDismantleDB( TAreaScan* );

private:
	 //! zugehoeriges AreaScanfenster
	 TAreaScan* Areascan;

	 //! realisiert test der Dialogboxinhalte auf ihre konsistenz vor verlassen
	 //! der Dialogbox (durch OK-Button)
	 //! Rueckkehrcode False wenn die Dlg.box nicht geschlossen werden kann
	 //! (Werte inkonsistent) sonst true
	 BOOL CanClose( void );

	 //! behandelt alle kommandos der Dialogbox
	 void Dlg_OnCommand( HWND, int, HWND, UINT );

	 //! setzt Defaultwerte der Dialogbox bei Aufruf der Box
	 //! Rueckkehrcode immer true
	 BOOL Dlg_OnInit( HWND, HWND, LPARAM );
};



#endif