//## Diese Kommentare (mit zwei Doppelkreuzen) sind von Marlies Gollnick ~:)

#ifndef __M_STEERG_H
#define __M_STEERG_H

// File m_steerg.h
// (C) 1993, 1994 by Heiko Damerow MPG AG "Roentgenbeugung"


#define   nMaxArg   4


// Steuer-Codes fuer Commands
typedef enum 
{ 
  CReady = 1, CHalted, CMeasure, CRecall, CReadyStep, 
      CFirstStep, CControlStep, CStop, CFailure
} TCCode;


//*************************************************************
class  TCmd
{
 friend class TSteering;

 public:
                 TCmd( TCmdTag );
  virtual        TCmd::~TCmd( void );
  TCCode         DoAction( void );
  virtual TCCode FirstStep( void )
      { return CReady; };
  virtual TCCode WakeUp( void );
  virtual BOOL   DeviceRequest( void );
  virtual BOOL   GetShowData( LPSTR )  
      { return TRUE; };
  virtual TCCode ControlStep( void )   
      { return CReady; };
  virtual TCCode ReadyStep( void )    
      { return CReady; };
  virtual TCCode Ready( void )      
      { return CReady; };
  BOOL           IsPositionValid( void ) 
      { return bPositionValid; };
  BOOL           IsReady( void )     
      { return ( eStep == CReady ); };
  TCmdId         GetId( void ) 
      { return eCmdId; };
  BOOL           StartMove( const int, double );
  virtual LPSTR  GetFailure( void );
 protected:
  TCCode         eStep;
  BOOL           bNoMeasure;   // normaly starts the device after moving
  BOOL*          bMoveOk;
  int            nFailureId;
  //! FIX Fehler 72
  BOOL          bRestart;
 private:
  TCmdId         eCmdId;
  BOOL           bPositionValid;
};


//*****************************************************************************
class  TLoadPointCmd : public TCmd
{
 public:
       TLoadPointCmd( TCmdTag );
  BOOL GetShowData( LPSTR );
};

class  TSetFileNameCmd : public TCmd
{
 public:
             TSetFileNameCmd( TCmdTag );
  BOOL       GetShowData( LPSTR );
 private:
  TScan*     ScanW;
  TAreaScan* AreaScanW;
};

class  TSaveDataCmd : public TCmd
{
 public:
             TSaveDataCmd( TCmdTag );
  BOOL       GetShowData( LPSTR );
 private:
  TScan*     ScanW;
  TAreaScan* AreaScanW;
};

class  TChooseAxisCmd : public TCmd
{
 public:
       TChooseAxisCmd( TCmdTag );
  BOOL GetShowData( LPSTR );
};

class  TSetupScanCmd : public TCmd
{
 public:
         TSetupScanCmd( TCmdTag );
  BOOL   GetShowData( LPSTR );
 private:
  TScan* ScanW;
};

class  TChooseDeviceCmd : public TCmd
{
 public:
       TChooseDeviceCmd( TCmdTag );
  BOOL GetShowData( LPSTR );
};

class  TSetWidthCmd : public TCmd
{
 public:
       TSetWidthCmd( TCmdTag );
  BOOL GetShowData( LPSTR );
};

class  TInquireCmd : public TCmd
{
 public:
    TInquireCmd( TCmdTag );
  BOOL      GetShowData( LPSTR );
};

class  TShowValueCmd : public TCmd
{
 public:
       TShowValueCmd( TCmdTag );
  BOOL GetShowData( LPSTR );
};

class  TCalculateCmd : public TCmd
{
 public:
       TCalculateCmd( TCmdTag );
  BOOL GetShowData( LPSTR );
};

class  TMoveToPointCmd : public TCmd
{
 public:
         TMoveToPointCmd( TCmdTag );
  BOOL   GetShowData( LPSTR );
  TCCode ReadyStep( void );
};

//! kommentiert von der Diffr.-gruppe
//! Ablaufsteuerung fuer StepScan und AreaScan(mit Psd)
class  TScanCmd : public TCmd
{
  public:
    TScanCmd( TCmdTag );

    //! setzt Initialwerte in erstem Schritt des Scans
    //! Rueckkehrcode immer CMeasure
    TCCode FirstStep( void );

    //! wird vom 2. bis zum letztem Schritt des Scans aufgerufen
    //! Rueckkehrcode immer CRecall
    TCCode ControlStep( void );

    //! wird nach letztem Schritt aufgerufen
    //! Rueckkehrcode immer CRecall
    TCCode ReadyStep( void );

    //! taetigt ausgaben in die Statusleiste waehrend und bei ende des Scans
    //! Rueckkehrcode immer True
    BOOL GetShowData( LPSTR );

  private:

    //! gibt Scanart an (ForScan = StepScan/ ForAreaScan = AreaScan)
    TCParam eCaller;

    //! gibt Scantyp an (Standard,Omega2Theta)
    TxScanType eScanType;

    //! Pointer auf zugehoeriges Scanfenster
    TScan* ScanW;

    //! Pointer auf zugehoeriges AreaScanfenster
    TAreaScan* AreaScanW;

    //! Anzahl schon gemessener Punkte
    int nPointIdx,

    //! Anzahl der gesamten zu messenden Punkte
    nPointNumber;

    //! Nummer des akt. Motors
    int nMotor,

    //! Nummer vom Theta-motor (nur bei Omega2Theta verwendet)
    nTheta,

    //! ?? (nur bei AreaScan verwendet)
    nAbsorber;

    //! Startwinkel des Scans
    double dArgumentMin,

    //! Endwinkel des Scans
    dArgumentMax,

    //! normale Schrittweite waehrend Scan
    dArgumentWidth;

    //! Startwinkel des Thetamotors
    double dThetaMin,

    //! Endwinkel des Thetamotors
    dThetaMax,

    //! Schrittweite des Thetamotors
    //! ??(alle 3 nur bei AreaScan verwendet)
    dThetaWidth;

    //! ?? winkel von Theta bei Scanstart (nur bei Omega2Theta verwendet)
    double dStartPointTheta,

    //! ? winkel des Absorber bei Scanstart(nur bei AreaScan verwendet)
    dStartPointAbsorber;

    //! gibt an ob Omega2Theta-Scan
    BOOL bOmega2Theta;

    //! gibt an ob Absorber benutzt
    BOOL bAbsorberUsed;

    //! gibt verhaeltnis an um das der Thetamotor bewegt werden soll, wenn sich
    //! der Omegamotor um einen Schritt bewegt (nur bei Omega2Theta verwendet)
    double dMoveRelation;

    //! gibt waehrend des Scans den naechsten anzufahrenden Winkel
    //! (Omega bei O2T-Scan)an
    double dArgument;

    //! ?? scheinbar ueberfluessig
    float fDriveInIntensity;

    //! ?? scheinbar ueberfluessig
    float fDriveOutIntensity;

    //! ?? scheinbar ueberfluessig
    float fIntensityRelation;

    //! ?? scheinbar ueberfluessig
    BOOL bAbsorberActivated;

    //! ?? scheinbar ueberfluessig
	 BOOL bAbsorberMovedIn;

	 //! gibt an ob dynamische Schrittweitensteuerung aktiv(nur bei StepScan
	 //! verwendet)
	 BOOL bDynamicStepWidth;

	 //! Anzahl der verschied. Schrittweiten die benutzt werden
	 //! (nur bei StepScan..)
	 int nStepScaleLevels;

	 //! Zeiger auf Liste mit Inten. und zugeh. Schrittweiten
	 float* Scaler;

   //! gibt waehrend des Scans den naechsten anzufahrenden Winkel
   //! (Theta bei O2T-Scan)an
   double dArgument2;

   //! gibt fuer den Areascan im standardmodus an ob ein psd-offset verwendet
   //! werden soll in dem Fall muss eine startposition angefahren werden ..
   BOOL bChannelOffset;
};

class  TGotoPeakCmd  : public TCmd
{
 public:
          TGotoPeakCmd( TCmdTag );
          ~TGotoPeakCmd();
  TCCode  FirstStep( void );
  TCCode  ControlStep( void );
  TCCode  ReadyStep( void );
  BOOL    GetShowData( LPSTR );
 private:
  int     nMotor;
  BOOL    bSmallAngleSide;
  BOOL    bFlankDetected;
  BOOL    bDynamicStepWidth;
  BOOL    bDirectionChanged;
  float   fSearchIntensity;
  TCParam nReadyAction;
  int     nLifo;
  double* dDistance;
  float*  fIntensity;
  BOOL    bBackMoveActive;
};

class  TControlFlankCmd : public TCmd
{
 public:
          TControlFlankCmd( TCmdTag );
          ~TControlFlankCmd();
  TCCode  FirstStep( void );
  TCCode  ControlStep( void );
  TCCode  ReadyStep( void );
  BOOL    GetShowData( LPSTR );
 private:
  int     nMotor;
  float   fControlRange;
  float   fStartIntensity, fSearchIntensity;
  float   fMinIntensity, fMaxIntensity, fIntensityRange;
  BOOL    bSmallAngleSide;
  int     nLifo;
  double* dDistance;
  float*  fIntensity;
};

class  TGotoIntensityCmd  : public TCmd
{
 public:
          TGotoIntensityCmd( TCmdTag );
          ~TGotoIntensityCmd();
  TCCode  FirstStep( void );
  TCCode  ControlStep( void );
  TCCode  ReadyStep( void );
  BOOL    GetShowData( LPSTR );
 private:
  int     nMotor;
  BOOL    bSmallAngleSide;
  float   fSearchIntensity;
  int     nLifo;
  double* dDistance;
  float*  fIntensity;
  TCParam nReadyAction;
};

//! kommentiert von der Diffr.-gruppe
//! Ablaufsteuerung fuer AreaScan(mit 0.dim. Detektor)SLD
class  TAreaScanCmd : public TCmd
{
  public:
         TAreaScanCmd( TCmdTag );

         //! aufruf aus DoAction() bei erstem Schritt des Scans
         //! Rueckkehrcode immer CMeasure
         TCCode FirstStep( void );

         //! aufruf aus DoAction() waehrend Scan
         //! Rueckkehrcode immer CRecall
         TCCode ControlStep( void );

         //! aufruf aus DoAction() wenn scan fertig ist
         //! Rueckkehrcode immer CRecall
         TCCode ReadyStep( void );

         //! realisiert die Ausgaben in die Statusleiste waehrend der Scan laeuft
         //! (sollte eigentlich nur den Ausg.string liefern die ausgabe sollte dann
         //! mit SendReport() getaetigt werden)
         //! 1. Param. Buffer enhaelt nach aufruf den ausgegeben String der Statusl.
         //! Rueckkehrcode immer true
         BOOL GetShowData( LPSTR );

  private:

         //! Pointer auf zugehoeriges Areascanfenster
         TAreaScan* AreaScanW;

         //! Nummern des Omega- bzw. Thetamotors
         int nOmega, nTheta;

         //! Thetagesamtbereich und Thetaschrittweite
         double dThetaWindow, dThetaWidth;

         //! Omegastart- , Omegaendposition sowie Omegaschrittweite
         double dOmegaMin, dOmegaMax, dOmegaWidth;

         //! Omega-,Thetapos. vor Scanstart
         double dStartPointTheta, dStartPointOmega;

         //! gibt verhaeltnis an um das der Thetamotor bewegt werden soll, wenn sich
         //! der Omegamotor um einen Schritt bewegt (nur bei Omega2Theta verwendet)
         double dMoveRelation;

         //! Omega-,Thetaposition der naechsten Messung
         double dOmega, dTheta;

         //! letzer bzw. akt. Schritt fuer Theta bzw. Omega
         int nMaxThetaPoint, nThetaPoint;
         int nMaxOmegaPoint, nOmegaPoint;

         //! enthaelt Wert fuer 2Theta-Startposition des aktuellen Spektrums
         double d2ThetaStart;
         };
//*****************************************************************************
//*****************************************************************************

class TEditWindow : public TMDIWindow
{
 public: 
          TEditWindow( LPSTR aFilename );
          ~TEditWindow();
  char    FileName[_MAX_PATH];
  BOOL    bIsNewFile;
  BOOL    bFileChanged;
  void    Create( HWND );
  BOOL    SetTitle( void );
  BOOL    IsModified( void ) 
      { return bFileChanged; };
  BOOL    CanClear( void );
  BOOL    NewFile( void );
  BOOL    SaveFile( int );
  BOOL    ReadFile( void );
  void    rButtonDown( LONG lParam );
  void    ClearModify() 
      {};
  HCURSOR GetCursor( void )
      { return LoadCursor( NULL, IDC_IBEAM ); };
  void    Size( HWND, WPARAM, LPARAM );
  void    Paint( HDC, PAINTSTRUCT * );
  void    SetFocus( void );
  void    ClearWindow( void )
      {};
  void    DoSearch( void )
      {};
  LPCSTR  ClassName( void ) 
      { return "Editor"; };
  LPCSTR  GetCharacteristic( void ) 
      { return "Edit"; };
};

class TAdjustmentParameter
{
  friend class TSetAdjustmentParam;
  friend class TAdjustmentExecute;
 public:         
         TAdjustmentParameter();
 protected:
  float  fNoiseLevel;
  float  fFailure;
  double dAngleRange;
};


#endif   
// __M_STEERG_H
// __LastLine__
