#include  "..\include\internls\evrythng.h"

#include  "..\include\motrstrg\motrstrg.h"

#include  "..\include\motrstrg\m_layer.h"
#include  "..\include\motrstrg\msimstat.h"

#include <fstream.h>

static HINSTANCE theSimulationDLL;
static ofstream*  theLogFile;

TMSimStatus::TMSimStatus()
{
  TMDIWindow::RegisterWnd();
  strcpy(Title, "MSIM-Status");
}

TMSimStatus::~TMSimStatus()
{
  // disconnect the simulation lib from the window   
  if ( theSimulationDLL ) {
    void (WINAPI* wndf)(HWND) = (void (WINAPI*)(HWND))
      GetProcAddress(theSimulationDLL, "SETSIMULATIONSTATUSWINDOW");
    if (wndf) wndf(0);
  }
}

//#pragma argsused
void TMSimStatus::Size(HWND hwnd, WPARAM wparam, LPARAM lparam)
{
  RECT rect;
  GetClientRect(hwnd, &rect);
  MoveWindow(m_StatusBox,
             0, 0, rect.right-rect.left, rect.bottom-rect.top,
             TRUE);
}

void TMSimStatus::Create(HWND hwnd)
{
  RECT rect;

  // set a proper height and width
  SetWindowPos(hwnd, (HWND)0, 0, 0, 250, 400, 0);

  // create the contained edit box
  GetClientRect(hwnd, &rect);
  m_StatusBox = CreateWindow(
                  "EDIT", "",
                  WS_CHILD|WS_VISIBLE|WS_VSCROLL|WS_HSCROLL|
                  ES_AUTOHSCROLL|ES_AUTOVSCROLL|ES_MULTILINE|ES_WANTRETURN,
                  0, 0, rect.right-rect.left, rect.bottom-rect.top,
                  hwnd, NULL,
                  0, 0l);
}



void InitializeMotorsSimulation(const char* ini_file)
{
  char buffer[100]; TSimulationType sim_type; int logLevel;

  // check whether we need to initialize the simulation

  GetPrivateProfileString("MOTORSIM", "SimulationType", "no_simulation",
      buffer, 100, ini_file);
  if (stricmp(buffer, "no_simulation") == 0)
  {
    sim_type = no_simulation;
  }
  else if (stricmp(buffer, "test_simulation") == 0)
  {
    sim_type = test_simulation;
  }
  else if (stricmp(buffer, "simulation_only") == 0)
  {
    sim_type = simulation_only;
  }


  // load simulation dll
  GetPrivateProfileString("MOTORSIM", "dll", "msim.dll",
                            buffer, 100, ini_file);
  HINSTANCE hLib = LoadLibrary(buffer);
  if (hLib == 0)
  {
    // library not found, reset no_simulation
    sim_type = no_simulation;
    return;
  }

  msSetSimulationType(sim_type);

  // open logstream if the name is given in ini-file
  GetPrivateProfileString("MOTORSIM", "LogFile", "msim.log",
                          buffer, 100, ini_file);
  logLevel = GetPrivateProfileInt("MOTORSIM", "LogLevel", 0, ini_file);
  if ( (strlen(buffer) > 0) && (logLevel > 0) )
  {
    theLogFile = new ofstream(buffer);

    void (WINAPI* setlogstream)(ostream*) = (void (WINAPI*)(ostream*))
        GetProcAddress(hLib, "SETLOGSTREAM");
    if(setlogstream) setlogstream(theLogFile);
  }


  // create the status window if required
  if ( sim_type != no_simulation &&
       GetPrivateProfileInt("MOTORSIM", "StatusWindow", 0, ini_file))
  {
      TMSimStatus* stat_wnd = new TMSimStatus();
      NewWindow((TMDIWindow *)stat_wnd);

      void (WINAPI* wndf)(HWND) = (void (WINAPI*)(HWND))
          GetProcAddress(hLib, "SETSIMULATIONSTATUSWINDOW");
      if (wndf) wndf(stat_wnd -> get_StatusBox());
  }


  // register the callbacks provided by msim.dll
  if ( logLevel > 0)
  {
    switch (sim_type) {
      case no_simulation:
        msRegister_C812ISA_Get( (T812ISA_GET_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C812ISA_GET_CALLBACK_NS") );
        msRegister_C812ISA_Put( (T812ISA_PUT_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C812ISA_PUT_CALLBACK") );
        msRegister_C832_Get( (T832_GET_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C832_GET_CALLBACK_NS") );
        msRegister_C832_Put( (T832_PUT_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C832_PUT_CALLBACK") );
        break;
      case test_simulation:
        msRegister_C812ISA_Get( (T812ISA_GET_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C812ISA_GET_CALLBACK_TS") );
        msRegister_C812ISA_Put( (T812ISA_PUT_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C812ISA_PUT_CALLBACK") );
        msRegister_C832_Get( (T832_GET_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C832_GET_CALLBACK_TS") );
        msRegister_C832_Put( (T832_PUT_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C832_PUT_CALLBACK") );
        break;
      case simulation_only:
        msRegister_C812ISA_Get( (T812ISA_GET_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C812ISA_GET_CALLBACK_SO") );
        msRegister_C812ISA_Put( (T812ISA_PUT_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C812ISA_PUT_CALLBACK") );
         msRegister_C832_Get( (T832_GET_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C832_GET_CALLBACK_SO") );
        msRegister_C832_Put( (T832_PUT_CALLBACK)
            GetProcAddress(hLib, "VERBOSE_C832_PUT_CALLBACK") );
        break;
    }
  }
  else
  {
    // no log requested, so we use silent callbacks, but only if we simulate
    if ( sim_type != no_simulation )
    {
      msRegister_C812ISA_Get( (T812ISA_GET_CALLBACK)
            GetProcAddress(hLib, "SILENT_C812ISA_GET_CALLBACK") );
      msRegister_C812ISA_Put( (T812ISA_PUT_CALLBACK)
            GetProcAddress(hLib, "SILENT_C812ISA_PUT_CALLBACK") );
      msRegister_C832_Get( (T832_GET_CALLBACK)
            GetProcAddress(hLib, "SILENT_C832_GET_CALLBACK") );
      msRegister_C832_Put( (T832_PUT_CALLBACK)
            GetProcAddress(hLib, "SILENT_C832_PUT_CALLBACK") );
    }
  }


  if ( sim_type != no_simulation )
  {
    void (WINAPI* initf)(const char*) = (void (WINAPI*)(const char*))
        GetProcAddress(hLib, "INITCONTROLLERFROMINIFILE");
    if (initf) initf(ini_file);
  }

  theSimulationDLL = hLib;
}


void UnInitializeMotorsSimulation()
{
  if (theSimulationDLL)
  {
    FreeLibrary(theSimulationDLL);
  }

  if (theLogFile)
  {
    theLogFile->close();
    delete theLogFile;
  }
}

