
#ifndef MC_832_H
#define MC_832_H

#include <iostream.h>
#include <assert.h>
#include <windows.h>
#include <mmsystem.h>

#ifdef __BCPLUSPLUS__
#define bool int
#define true 1
#define false 0
#endif

#include "../msimutil.h"
#include "../export.h"

class MSIMAPI MC_832
{
  public:
    enum target_register { cmd_reg = 0, data_reg = 1, int_reg = 2 };

    MC_832( );
    ~MC_832( );

    void SetLogStream( ostream* );
    ostream* GetLogStream( );
    void SetLogLevel( int );
    int  GetLogLevel( );
    const char* GetStatusText( );
    void UpdateStatusText( );
    void UpdateStatus( );
    void SetIOAddress( unsigned );
    unsigned GetIOAddress( );
    void SetHysteresis( int board_id, long value );
    long GetHysteresis( int board_id );
    void SetLastDirection( int board_id, bool direction ); // true = -->
    bool GetLastDirection( int board_id );
    void SetPhysicalPositionRange( int board_id, long val );
    long GetPhysicalPositionRange( int board_id );
    void SetPhysicalPosition( int board_id, long val );
    long GetPhysicalPosition( int board_id );
    void SetPhysicalIndexPosition( int board_id, long val );
    long GetPhysicalIndexPosition( int board_id );

    void Put( unsigned port, byte value );
    byte Get( unsigned port );

    // returns the time in milliseconds since construction
    long GetCurrentTime( );

    // base class for all commands
    class Command;
    // a class for every command which has a state (i.e. a command
    // which accepts parameters or returns values)
    class Cmd16BitReport;
    class Cmd32BitReport;
    class CmdLTRJ;
    class CmdMSKI;
    class CmdRSTI;
    class CmdLFIL;
    class CmdRDIP;
    class CmdRDRP;
    class CmdRDDP;
    class CmdRDSIGS;

    class Motor;

  private:
    Motor* m_motor[ 2 ];
    int m_current_motor;

    // I/O-address of the addressregister
    unsigned m_addr_reg_addr;
    // I/O-address of the dataregister
    unsigned m_data_reg_addr;
    // determine what register is to be read or written (determined by the last
    // value written to the addressregister)
    target_register m_target_register;

    ostream* m_log_stream;
    int m_log_level;

    // stores the construction point time
    //timeb m_starttime;
    long m_starttime;

    char m_status_text[ 1024 ];
};

#ifdef __BCPLUSPLUS__
  MC_832* MSIMAPI getTheC832();
#else
  MSIMAPI MC_832* getTheC832();
#endif

inline
void MC_832::SetIOAddress( unsigned addr )
{
  m_addr_reg_addr = addr;
  m_data_reg_addr = addr + 1;
}

inline
unsigned MC_832::GetIOAddress( )
{
  return m_addr_reg_addr;
}


inline
void MC_832::SetLogStream( ostream* os )
{
  m_log_stream = os;
}

inline
ostream* MC_832::GetLogStream( )
{
  return m_log_stream;
}

inline
void MC_832::SetLogLevel( int lvl )
{
  m_log_level = lvl;
}

inline
int MC_832::GetLogLevel( )
{
  return m_log_level;
}

inline
long MC_832::GetCurrentTime( )
{
  return timeGetTime() - m_starttime;
}


#endif MC_832_H

